@@ -112,7 +112,6 @@ EXCLUDE = (
     "lib/ast/fmtesc.c",
     "lib/ast/stresc.c",
     "lib/ast/strmatch.c",
-    "lib/cdt/cdt.h",
     "lib/cdt/dtclose.c",
     "lib/cdt/dtdisc.c",
     "lib/cdt/dtextract.c",
@@ -462,13 +461,6 @@ EXCLUDE = (
     "lib/twopigen/circle.c",
     "lib/twopigen/circle.h",
     "lib/twopigen/twopiinit.c",
-    "lib/vmalloc/test.c",
-    "lib/vmalloc/vmalloc.c",
-    "lib/vmalloc/vmalloc.h",
-    "lib/vmalloc/vmclear.c",
-    "lib/vmalloc/vmclose.c",
-    "lib/vmalloc/vmopen.c",
-    "lib/vmalloc/vmstrdup.c",
     "lib/vpsc/block.cpp",
     "lib/vpsc/block.h",
     "lib/vpsc/blocks.cpp",
