@@ -16,6 +16,7 @@
 #include "gltemplate.h"
 #include <glcomp/glutils.h>
 #include <util/exit.h>
+#include <util/list.h>
 #include "glexpose.h"
 #include "glmotion.h"
 
@@ -66,7 +67,8 @@ static void realize(GtkWidget *widget, void *data) {
     if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
 	return;
 
-    glClearColor(view->bgColor.R, view->bgColor.G, view->bgColor.B, view->bgColor.A);	//background color
+    glClearColor((float)view->bgColor.R, (float)view->bgColor.G,
+                 (float)view->bgColor.B, (float)view->bgColor.A); // background color
     glClearDepth(1.0);
     glClear(GL_COLOR_BUFFER_BIT);
 
@@ -290,7 +292,7 @@ static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event,
 	appmouse_drag(view,(int)event->x,(int)event->y);
 	redraw = true;
     }
-    if (!glCompPoly_is_empty(&view->Topview->sel.selPoly))
+    if (!LIST_IS_EMPTY(&view->Topview->sel.selPoly))
 	redraw = true;
 
     begin_x = x;
