@@ -34,10 +34,9 @@
 #include <util/streq.h>
 
 struct adjmatrix_t {
-  size_t nrows;
-  size_t ncols;
-  uint8_t *data;    ///< bit-packed backing memory
-  size_t allocated; ///< how many bytes have been allocated backing `data`?
+  size_t nrows;  ///< how many rows have been allocated?
+  size_t ncols;  ///< how many columns have been allocated?
+  uint8_t *data; ///< bit-packed backing memory
 };
 
 /// get the value of a matrix cell
@@ -49,14 +48,17 @@ struct adjmatrix_t {
 static bool matrix_get(adjmatrix_t *me, size_t row, size_t col) {
   assert(me != NULL);
 
-  // if this index is beyond anything set, infer it as unset
-  const size_t index = row * me->ncols + col;
-  const size_t byte_index = index / 8;
-  const size_t bit_index = index % 8;
-  if (byte_index >= me->allocated) {
+  // if this index is beyond anything allocated, infer it as unset
+  if (row >= me->nrows) {
+    return false;
+  }
+  if (col >= me->ncols) {
     return false;
   }
 
+  const size_t index = row * me->ncols + col;
+  const size_t byte_index = index / 8;
+  const size_t bit_index = index % 8;
   return (me->data[byte_index] >> bit_index) & 1;
 }
 
@@ -69,15 +71,38 @@ static void matrix_set(adjmatrix_t *me, size_t row, size_t col) {
   assert(me != NULL);
 
   // if we are updating beyond allocated space, expand the backing store
+  if (row >= me->nrows || col >= me->ncols) {
+    // allocate an enlarged space
+    const size_t nrows = zmax(me->nrows, row + 1);
+    const size_t ncols = zmax(me->ncols, col + 1);
+    const size_t bits = nrows * ncols;
+    const size_t bytes = bits / 8 + (bits % 8 == 0 ? 0 : 1);
+    uint8_t *const data = gv_alloc(bytes);
+
+    // replicate set bits
+    for (size_t r = 0; r < me->nrows; ++r) {
+      for (size_t c = 0; c < me->ncols; ++c) {
+        if (!matrix_get(me, r, c)) {
+          continue;
+        }
+        const size_t index = r * ncols + c;
+        const size_t byte_index = index / 8;
+        const size_t bit_index = index % 8;
+        data[byte_index] |= (uint8_t)(UINT8_C(1) << bit_index);
+      }
+    }
+
+    // replace old matrix with newly expanded one
+    free(me->data);
+    *me = (adjmatrix_t){.nrows = nrows, .ncols = ncols, .data = data};
+  }
+
+  assert(row < me->nrows);
+  assert(col < me->ncols);
+
   const size_t index = row * me->ncols + col;
   const size_t byte_index = index / 8;
   const size_t bit_index = index % 8;
-  if (byte_index >= me->allocated) {
-    me->data = gv_recalloc(me->data, me->allocated, byte_index + 1,
-                           sizeof(me->data[0]));
-    me->allocated = byte_index + 1;
-  }
-
   me->data[byte_index] |= (uint8_t)(UINT8_C(1) << bit_index);
 }
 
@@ -105,7 +130,17 @@ static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
 static void save_best(graph_t * g);
 static void restore_best(graph_t * g);
-static adjmatrix_t *new_matrix(size_t i, size_t j);
+
+/// create an adjacency matrix
+///
+/// These matrices dynamically expand on demand. So the initial dimensions
+/// provided can be later safely exceeded at runtime.
+///
+/// @param initial_rows How many rows to allocate now
+/// @param initial_columns How many columns to allocate now
+/// @return A constructed matrix
+static adjmatrix_t *new_matrix(size_t initial_rows, size_t initial_columns);
+
 static void free_matrix(adjmatrix_t * p);
 static int ordercmpf(const void *, const void *);
 static int64_t ncross(void);
@@ -148,15 +183,14 @@ static void nname(node_t *v, FILE *stream) {
 }
 static void dumpg (graph_t* g)
 {
-    int j, i, r;
-    node_t* v;
+    int i;
     edge_t* e;
 
     fprintf (stderr, "digraph A {\n");
-    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+    for (int r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	fprintf (stderr, "  subgraph {rank=same  ");
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
+	  node_t *const v = GD_rank(g)[r].v[i];
           if (i > 0)
  	    fputs(" -> ", stderr);
           nname(v, stderr);
@@ -164,10 +198,10 @@ static void dumpg (graph_t* g)
         if (i > 1) fprintf (stderr, " [style=invis]}\n");
         else fprintf (stderr, " }\n");
     }
-    for (r = GD_minrank(g); r < GD_maxrank(g); r++) {
+    for (int r = GD_minrank(g); r < GD_maxrank(g); r++) {
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
-	  for (j = 0; (e = ND_out(v).list[j]); j++) {
+	  node_t *const v = GD_rank(g)[r].v[i];
+	  for (int j = 0; (e = ND_out(v).list[j]); j++) {
              nname(v, stderr);
              fputs(" -> ", stderr);
              nname(aghead(e), stderr);
@@ -444,10 +478,12 @@ int dot_mincross(graph_t *g) {
     return 0;
 }
 
-static adjmatrix_t *new_matrix(size_t i, size_t j) {
+static adjmatrix_t *new_matrix(size_t initial_rows, size_t initial_columns) {
     adjmatrix_t *rv = gv_alloc(sizeof(adjmatrix_t));
-    rv->nrows = i;
-    rv->ncols = j;
+    const size_t bits = initial_rows * initial_columns;
+    const size_t bytes = bits / 8 + (bits % 8 == 0 ? 0 : 1);
+    uint8_t *const data = gv_alloc(bytes);
+    *rv = (adjmatrix_t){.nrows = initial_rows, .ncols = initial_columns, .data = data};
     return rv;
 }
 
@@ -1137,8 +1173,6 @@ static void flat_search(graph_t * g, node_t * v)
 	    if (ED_weight(e) == 0)
 		continue;
 	    if (ND_onstack(aghead(e))) {
-		assert(flatindex(aghead(e)) < M->nrows);
-		assert(flatindex(agtail(e)) < M->ncols);
 		matrix_set(M, (size_t)flatindex(aghead(e)), (size_t)flatindex(agtail(e)));
 		delete_flat_edge(e);
 		i--;
@@ -1146,8 +1180,6 @@ static void flat_search(graph_t * g, node_t * v)
 		    continue;
 		flat_rev(g, e);
 	    } else {
-		assert(flatindex(aghead(e)) < M->nrows);
-		assert(flatindex(agtail(e)) < M->ncols);
 		matrix_set(M, (size_t)flatindex(agtail(e)), (size_t)flatindex(aghead(e)));
 		if (!ND_mark(aghead(e)))
 		    flat_search(g, aghead(e));
@@ -1302,26 +1334,26 @@ int build_ranks(graph_t *g, int pass) {
 	    continue;
 	if (!MARK(n)) {
 	    MARK(n) = true;
-	    node_queue_push_back(&q, n);
-	    while (!node_queue_is_empty(&q)) {
-		node_t *n0 = node_queue_pop_front(&q);
+	    LIST_PUSH_BACK(&q, n);
+	    while (!LIST_IS_EMPTY(&q)) {
+		node_t *n0 = LIST_POP_FRONT(&q);
 		if (ND_ranktype(n0) != CLUSTER) {
 		    if (install_in_rank(g, n0) != 0) {
-		        node_queue_free(&q);
+		        LIST_FREE(&q);
 		        return -1;
 		    }
 		    enqueue_neighbors(&q, n0, pass);
 		} else {
 		    const int rc = install_cluster(g, n0, pass, &q);
 		    if (rc != 0) {
-		        node_queue_free(&q);
+		        LIST_FREE(&q);
 		        return rc;
 		    }
 		}
 	    }
 	}
     }
-    assert(node_queue_is_empty(&q));
+    assert(LIST_IS_EMPTY(&q));
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 	GD_rank(Root)[i].valid = false;
 	if (GD_flip(g) && GD_rank(g)[i].n > 0) {
@@ -1335,7 +1367,7 @@ int build_ranks(graph_t *g, int pass) {
 
     if (g == dot_root(g) && ncross() > 0)
 	transpose(g, false);
-    node_queue_free(&q);
+    LIST_FREE(&q);
     return 0;
 }
 
@@ -1347,7 +1379,7 @@ void enqueue_neighbors(node_queue_t *q, node_t *n0, int pass) {
 	    e = ND_out(n0).list[i];
 	    if (!MARK(aghead(e))) {
 		MARK(aghead(e)) = true;
-		node_queue_push_back(q, aghead(e));
+		LIST_PUSH_BACK(q, aghead(e));
 	    }
 	}
     } else {
@@ -1355,7 +1387,7 @@ void enqueue_neighbors(node_queue_t *q, node_t *n0, int pass) {
 	    e = ND_in(n0).list[i];
 	    if (!MARK(agtail(e))) {
 		MARK(agtail(e)) = true;
-		node_queue_push_back(q, agtail(e));
+		LIST_PUSH_BACK(q, agtail(e));
 	    }
 	}
     }
@@ -1371,7 +1403,7 @@ static bool constraining_flat_edge(Agraph_t *g, Agedge_t *e) {
   return true;
 }
 
-DEFINE_LIST(nodes, node_t *)
+typedef LIST(node_t *) nodes_t;
 
 /* construct nodes reachable from 'here' in post-order.
 * This is the same as doing a topological sort in reverse order.
@@ -1389,7 +1421,7 @@ static void postorder(graph_t *g, node_t *v, nodes_t *list, int r) {
 	}
     }
     assert(ND_rank(v) == r);
-    nodes_append(list, v);
+    LIST_APPEND(list, v);
 }
 
 static void flat_reorder(graph_t * g)
@@ -1406,7 +1438,7 @@ static void flat_reorder(graph_t * g)
 	base_order = ND_order(GD_rank(g)[r].v[0]);
 	for (i = 0; i < GD_rank(g)[r].n; i++)
 	    MARK(GD_rank(g)[r].v[i]) = false;
-	nodes_clear(&temprank);
+	LIST_CLEAR(&temprank);
 
 	/* construct reverse topological sort order in temprank */
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
@@ -1423,7 +1455,7 @@ static void flat_reorder(graph_t * g)
 		if (constraining_flat_edge(g, flat_e)) local_out_cnt++;
 	    }
 	    if (local_in_cnt == 0 && local_out_cnt == 0)
-		nodes_append(&temprank, v);
+		LIST_APPEND(&temprank, v);
 	    else {
 		if (!MARK(v) && local_in_cnt == 0) {
 		    postorder(g, v, &temprank, r);
@@ -1431,12 +1463,12 @@ static void flat_reorder(graph_t * g)
 	    }
 	}
 
-	if (nodes_size(&temprank) > 0) {
+	if (!LIST_IS_EMPTY(&temprank)) {
 	    if (!GD_flip(g)) {
-		nodes_reverse(&temprank);
+		LIST_REVERSE(&temprank);
 	    }
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
-		v = GD_rank(g)[r].v[i] = nodes_get(&temprank, (size_t)i);
+		v = GD_rank(g)[r].v[i] = LIST_GET(&temprank, (size_t)i);
 		ND_order(v) = i + base_order;
 	    }
 
@@ -1460,7 +1492,7 @@ static void flat_reorder(graph_t * g)
 	/* else do no harm! */
 	GD_rank(Root)[r].valid = false;
     }
-    nodes_free(&temprank);
+    LIST_FREE(&temprank);
 }
 
 static void reorder(graph_t * g, int r, bool reverse, bool hasfixed)
