@@ -22,9 +22,8 @@
 #include <neatogen/info.h>
 #include <neatogen/edges.h>
 #include <neatogen/site.h>
-#include <neatogen/hedges.h>
 #include <neatogen/digcola.h>
-#if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
+#if defined(HAVE_GTS) && defined(SFDP)
 #include <neatogen/overlap.h>
 #endif
 #include <stdbool.h>
@@ -422,9 +421,7 @@ static void newPos(const state_t *st, bool doAll) {
  */
 static void cleanup(void)
 {
-    ELcleanup();
     siteinit();			/* free memory */
-    edgeinit();			/* free memory */
 }
 
 static int vAdjust(state_t *st) {
@@ -599,7 +596,7 @@ SparseMatrix makeMatrix(Agraph_t *g) {
     return A;
 }
 
-#if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
+#if defined(HAVE_GTS) && defined(SFDP)
 static void fdpAdjust(graph_t *g, adjust_data *am) {
     SparseMatrix A0 = makeMatrix(g);
     SparseMatrix A = A0;
@@ -791,7 +788,7 @@ typedef struct {
  */
 static const lookup_t adjustMode[] = {
     {AM_NONE, "", "none"},
-#if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
+#if defined(HAVE_GTS) && defined(SFDP)
     {AM_PRISM, "prism", "prism"},
 #endif
     {AM_VOR, "voronoi", "Voronoi"},
@@ -809,7 +806,7 @@ static const lookup_t adjustMode[] = {
     {AM_PORTHO_YX, "portho_yx", "pseudo-orthogonal constraints"},
     {AM_PORTHOXY, "porthoxy", "xy pseudo-orthogonal constraints"},
     {AM_PORTHOYX, "porthoyx", "yx pseudo-orthogonal constraints"},
-#if !((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
+#if !(defined(HAVE_GTS) && defined(SFDP))
     {AM_PRISM, "prism", 0},
 #endif
     {0}
@@ -956,7 +953,7 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 	case AM_COMPRESS:
 	    ret = scAdjust(G, -1);
 	    break;
-#if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
+#if defined(HAVE_GTS) && defined(SFDP)
 	case AM_PRISM:
 	    fdpAdjust(G, am);
 	    ret = 0;
