@@ -1,5 +1,5 @@
 /*************************************************************************
- * Copyright (c) 2011 AT&T Intellectual Property 
+ * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -16,7 +16,15 @@ extern "C" {
 
 #include <neatogen/defs.h>
 
-    extern void closest_pairs2graph(double *, int, int, vtx_data **);
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
+#endif
+
+INTERNAL void closest_pairs2graph(double *, int, int, vtx_data **);
+
+#undef INTERNAL
 
 #ifdef __cplusplus
 }
