@@ -1,9 +1,9 @@
 /*************************************************************************
- * Copyright (c) 2011 AT&T Intellectual Property 
+ * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -16,9 +16,16 @@ extern "C" {
 
 #include <neatogen/defs.h>
 
-    extern void embed_graph(vtx_data * graph, int n, int dim, DistType ***,
-			    int);
-    extern void center_coordinate(DistType **, int, int);
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
+#endif
+
+INTERNAL void embed_graph(vtx_data *graph, int n, int dim, DistType ***, int);
+INTERNAL void center_coordinate(DistType **, int, int);
+
+#undef INTERNAL
 
 #ifdef __cplusplus
 }
