@@ -1,27 +1,25 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include <stddef.h>
 
-
-#ifndef MEMORY_H
-#define MEMORY_H
-
-#ifndef NULL
-#define NULL 0
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
 #endif
 
     /* Support for freelists */
@@ -32,12 +30,11 @@ extern "C" {
 	int nodesize;		/* Size of node */
     } Freelist;
 
-    extern void *getfree(Freelist *);
-    extern void freeinit(Freelist *, int);
-    extern void makefree(void *, Freelist *);
-
-#endif
+INTERNAL void *getfree(Freelist *);
+INTERNAL void freeinit(Freelist *, int);
+INTERNAL void makefree(void *, Freelist *);
 
+#undef INTERNAL
 
 #ifdef __cplusplus
 }
