@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -16,6 +16,12 @@ extern "C" {
 
 #include <neatogen/geometry.h>
 
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
+#endif
+
     typedef struct {
 	Point origin;
 	Point corner;
@@ -24,11 +30,13 @@ extern "C" {
 	int kind;
     } Poly;
 
-    extern void polyFree(void);
-    extern int polyOverlap(Point, Poly *, Point, Poly *);
-    extern int makePoly(Poly *, Agnode_t *, float, float);
-    extern int makeAddPoly(Poly *, Agnode_t *, float, float);
-    extern void breakPoly(Poly *);
+INTERNAL void polyFree(void);
+INTERNAL int polyOverlap(Point, Poly *, Point, Poly *);
+INTERNAL int makePoly(Poly *, Agnode_t *, double, double);
+INTERNAL int makeAddPoly(Poly *, Agnode_t *, double, double);
+INTERNAL void breakPoly(Poly *);
+
+#undef INTERNAL
 
 #ifdef __cplusplus
 }
